package com.example.fieldagent.ui.damagelist

import android.content.ContentValues
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.graphics.pdf.PdfDocument
import android.os.Bundle
import android.os.Environment
import android.provider.MediaStore
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.fragment.app.viewModels
import androidx.lifecycle.Observer
import androidx.navigation.fragment.findNavController
import com.consultantapp.data.network.responseUtil.Status
import com.example.fieldagent.R
import com.example.fieldagent.data.network.ApisRespHandler
import com.example.fieldagent.databinding.FragmentDamageListBinding
import com.example.fieldagent.ui.damagelist.adapter.DamageListAdapter
import com.example.fieldagent.utils.PrefsManager
import com.example.fieldagent.utils.dialogs.ProgressDialog
import com.example.fieldagent.utils.gone
import com.example.fieldagent.utils.hideShowView
import com.example.fieldagent.utils.showConfirmDialog
import com.example.fieldagent.utils.visible
import com.google.gson.Gson
import dagger.hilt.android.AndroidEntryPoint
import java.io.File
import java.io.FileOutputStream
import java.text.SimpleDateFormat
import java.util.Date
import java.util.HashMap
import java.util.Locale
import javax.inject.Inject


@AndroidEntryPoint
class DamageListFragment : Fragment() {

    private var _binding: FragmentDamageListBinding? = null

    private val binding get() = _binding!!

    private  val viewModel: DamageListFragmentVM by viewModels()

    @Inject
    lateinit var prefsManager: PrefsManager

    private lateinit var progressDialog: ProgressDialog

    private lateinit var adapter: DamageListAdapter

    private var inspectionId=""
    private var clickDeletePos=-1

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentDamageListBinding.inflate(inflater, container, false)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)


        initialize()
        listener()
        bindObservers()

    }

   private fun initialize(){

       progressDialog = ProgressDialog(requireActivity())
       adapter = DamageListAdapter(this)
       binding.rvDamageList.adapter = adapter

       inspectionId = arguments?.getString("id").toString()
       val name = arguments?.getString("name").toString()

       // Use the data
       name.let {
          binding.txtTitle.text=name
       }
       val hashMap = HashMap<String, String>()
       hashMap["id"] = inspectionId
       viewModel.getDamagesList(hashMap)
    }


   private fun bindObservers(){
       viewModel.damages.observe(requireActivity(), Observer {
           it ?: return@Observer
           when (it.status) {
               Status.SUCCESS -> {

                   progressDialog.setLoading(false)

                   val tempList = it.data?.data?.inspections ?: emptyList()

                   if(tempList[0].damages==null || tempList[0].damages!!.isEmpty())
                   {

                       binding.clNoData.visible()
                       binding.btnExportPdf.gone()
                   }else{
                       binding.clNoData.gone()
                       binding.btnExportPdf.visible()
                   }
                   tempList[0].damages?.let { it1 ->
                       adapter.submitList(it1)
                       binding.damageCount.text=String.format("%02d", it1.size)
                   }



               }
               Status.ERROR -> {
                   progressDialog.setLoading(false)
                   ApisRespHandler.handleError(it.error, requireActivity())
               }
               Status.LOADING -> {
                   progressDialog.setLoading(true)
               }
           }
       })

       viewModel.deleteDamage.observe(requireActivity(), Observer {
           it ?: return@Observer
           when (it.status) {
               Status.SUCCESS -> {
                   progressDialog.setLoading(false)
                   if(clickDeletePos!=-1){

                       adapter.mainList.removeAt(clickDeletePos)
                       adapter.notifyDataSetChanged()

                       clickDeletePos=-1

                       if(adapter.mainList.isEmpty())
                       {
                           binding.clNoData.visible()
                           binding.btnExportPdf.gone()
                       }else{
                           binding.clNoData.gone()
                           binding.btnExportPdf.visible()
                       }

                       binding.damageCount.text=String.format("%02d", adapter.mainList.size)

                   }

               }
               Status.ERROR -> {

                   progressDialog.setLoading(false)
                   ApisRespHandler.handleError(it.error, requireActivity())
               }
               Status.LOADING -> {
                   progressDialog.setLoading(true)
               }
           }
       })
    }

    private fun listener(){

        binding.llBtn.setOnClickListener{


            val bundle = Bundle().apply {
                putString("id", inspectionId)
                putString("from", "add")
            }
            findNavController().navigate(R.id.damageDeatilsFragment, bundle)
        }
        binding.ivLeft.setOnClickListener {

            findNavController().popBackStack()
        }

        binding.btnExportPdf.setOnClickListener {
            setUpPdf()
        }

    }

    private fun setUpPdf(){
        val pdfDocument = PdfDocument()
        val pageInfo = PdfDocument.PageInfo.Builder(595, 842, 1).create()
        val page = pdfDocument.startPage(pageInfo)
        val canvas = page.canvas

        val borderPaint = Paint().apply {
            color = Color.BLACK
            style = Paint.Style.STROKE
            strokeWidth = 1f
        }

        val textPaint = Paint().apply {
            textSize = 12f
            color = Color.BLACK
            style = Paint.Style.FILL
            isAntiAlias = true
        }

        val whiteTextPaint = Paint(textPaint).apply { color = Color.WHITE }

        val fillPaint = Paint().apply {
            style = Paint.Style.FILL
        }

// Column layout
        val colWidths = listOf(40f, 100f, 60f, 100f, 60f, 120f)
        val rowHeight = 40f
        val startX = 40f
        var startY = 60f

        val title = "Damage Report Table"
        val headers = listOf("No.", "Item", "Number", "Individual Cost", "Unit", "Total Cost (Riyals)")
        val rows = listOf(
            listOf("1", "Chair", "4", "50", "pcs", "200"),
            listOf("2", "Table", "2", "150", "pcs", "300"),
            listOf("3", "مكتب", "1", "500", "قطعة", "500"),
            listOf("4", "Desk", "1", "450", "pcs", "450")
        )

        val centerAlignedColumns = setOf(0, 2, 3, 5) // columns to center text

        fun drawCellText(canvas: Canvas, text: String, x: Float, y: Float, width: Float, paint: Paint, center: Boolean) {
            val drawX = if (center) {
                x + (width / 2f) - (paint.measureText(text) / 2f)
            } else {
                x + 5f
            }
            canvas.drawText(text, drawX, y, paint)
        }

// --- Draw title row ---
        fillPaint.color = Color.parseColor("#006400") // dark green
        canvas.drawRect(startX, startY, startX + colWidths.sum(), startY + rowHeight, fillPaint)
        canvas.drawRect(startX, startY, startX + colWidths.sum(), startY + rowHeight, borderPaint)

        val titleX = startX + (colWidths.sum() / 2f) - (whiteTextPaint.measureText(title) / 2f)
        canvas.drawText(title, titleX, startY + 25f, whiteTextPaint)
        startY += rowHeight

// --- Draw header row ---
        fillPaint.color = Color.parseColor("#90EE90") // light green
        var x = startX
        for ((i, header) in headers.withIndex()) {
            val width = colWidths[i]
            canvas.drawRect(x, startY, x + width, startY + rowHeight, fillPaint)
            canvas.drawRect(x, startY, x + width, startY + rowHeight, borderPaint)
            drawCellText(canvas, header, x, startY + 25f, width, textPaint, i in centerAlignedColumns)
            x += width
        }
        startY += rowHeight

// --- Draw data rows ---
        for ((index, row) in rows.withIndex()) {
            fillPaint.color = if (index % 2 == 0) Color.WHITE else Color.parseColor("#E6FFE6")
            x = startX
            for ((i, cell) in row.withIndex()) {
                val width = colWidths[i]
                canvas.drawRect(x, startY, x + width, startY + rowHeight, fillPaint)
                canvas.drawRect(x, startY, x + width, startY + rowHeight, borderPaint)
                drawCellText(canvas, cell, x, startY + 25f, width, textPaint, i in centerAlignedColumns)
                x += width
            }
            startY += rowHeight
        }

        pdfDocument.finishPage(page)


        val fileName = "damage_report_" + SimpleDateFormat("yyyyMMdd_HHmmss", Locale.getDefault()).format(
            Date()
        )
      savePdfToDocuments(pdfDocument, fileName)

    }
    fun savePdfToDocuments( pdfDocument: PdfDocument, fileName: String) {
        val contentValues = ContentValues().apply {
            put(MediaStore.MediaColumns.DISPLAY_NAME, "$fileName.pdf")
            put(MediaStore.MediaColumns.MIME_TYPE, "application/pdf")
            put(MediaStore.MediaColumns.RELATIVE_PATH, Environment.DIRECTORY_DOCUMENTS)
            put(MediaStore.MediaColumns.IS_PENDING, 1)
        }

        val resolver = requireActivity().contentResolver
        val uri = resolver.insert(MediaStore.Files.getContentUri("external"), contentValues)

        uri?.let {
            resolver.openOutputStream(it)?.use { outputStream ->
                pdfDocument.writeTo(outputStream)
                pdfDocument.close()
            }

            contentValues.clear()
            contentValues.put(MediaStore.MediaColumns.IS_PENDING, 0)
            resolver.update(uri, contentValues, null, null)

            Toast.makeText(context, "PDF saved to Documents!", Toast.LENGTH_LONG).show()
        } ?: run {
            Toast.makeText(context, "Failed to save PDF", Toast.LENGTH_SHORT).show()
        }
    }



    fun clickItem(pos: Int) {
        val gson = Gson()
        val jsonString = gson.toJson(adapter.mainList[pos])
        val bundle = Bundle().apply {
            putString("damageData", jsonString)
            putString("id", inspectionId)
            putString("from", "edit")
        }
        findNavController().navigate(R.id.damageDeatilsFragment, bundle)

    }
    fun clickDeleteItem(pos: Int, id: String,) {

        if (pos < 0 ) {
            return
        }
        clickDeletePos=pos


        requireContext().showConfirmDialog(
            title = getString(R.string.are_you_sure_you_want_to_delete)
        ) {
            viewModel.deleteInspection(id)
        }


    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}