package com.example.fieldagent.ui.homescreen

import androidx.lifecycle.ViewModel
import com.example.fieldagent.data.apis.WebService
import com.example.fieldagent.data.models.responses.InspectionData
import com.example.fieldagent.data.models.responses.SimpleResponseData
import com.example.fieldagent.data.network.responseUtil.ApiUtils
import com.example.fieldagent.data.network.responseUtil.Resource
import com.example.fieldagent.di.SingleLiveEvent
import dagger.hilt.android.lifecycle.HiltViewModel
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import javax.inject.Inject


@HiltViewModel
class HomeViewModel @Inject constructor(private val webService: WebService) : ViewModel() {

    val inspections by lazy { SingleLiveEvent<Resource<InspectionData>>() }
    val delete by lazy { SingleLiveEvent<Resource<SimpleResponseData>>() }

    fun getInspectionList(hashMap: HashMap<String, String>) {
        inspections.value = Resource.loading()

        webService.inspectionsList(hashMap)
            .enqueue(object : Callback<InspectionData> {

                override fun onResponse(call: Call<InspectionData>,
                                        response: Response<InspectionData>
                ) {
                    if (response.isSuccessful) {
                        inspections.value = Resource.success(response.body())

                    } else {
                        inspections.value = Resource.error(
                            ApiUtils.getError(response.code(),
                                response.errorBody()?.string()))
                    }
                }

                override fun onFailure(call: Call<InspectionData>, throwable: Throwable) {
                    inspections.value = Resource.error(ApiUtils.failure(throwable))
                }

            })
    }


    fun deleteInspection(id:String) {
        delete.value = Resource.loading()

        webService.deleteInspections(id)
            .enqueue(object : Callback<SimpleResponseData> {

                override fun onResponse(call: Call<SimpleResponseData>,
                                        response: Response<SimpleResponseData>
                ) {
                    if (response.isSuccessful) {
                        delete.value = Resource.success(response.body())

                    } else {
                        delete.value = Resource.error(
                            ApiUtils.getError(response.code(),
                                response.errorBody()?.string()))
                    }
                }

                override fun onFailure(call: Call<SimpleResponseData>, throwable: Throwable) {
                    delete.value = Resource.error(ApiUtils.failure(throwable))
                }

            })
    }


}